@@ -1,15 +1,14 @@
 #
 #  This file is part of WebDyne.
 #
-#  This software is Copyright (c) 2017 by Andrew Speer <andrew@webdyne.org>.
+#  This software is copyright (c) 2026 by Andrew Speer <andrew.speer@isolutions.com.au>.
 #
-#  This is free software, licensed under:
-#
-#    The GNU General Public License, Version 2, June 1991
+#  This is free software; you can redistribute it and/or modify it under
+#  the same terms as the Perl 5 programming language system itself.
 #
 #  Full license text is available at:
 #
-#  <http://www.gnu.org/licenses/old-licenses/gpl-2.0.txt>
+#  <http://dev.perl.org/licenses/>
 #
 package WebDyne::Err;
 
@@ -17,7 +16,7 @@ package WebDyne::Err;
 #  Compiler Pragma
 #
 use strict qw(vars);
-use vars qw($VERSION);
+use vars   qw($VERSION);
 use warnings;
 no warnings qw(uninitialized);
 
@@ -26,18 +25,20 @@ no warnings qw(uninitialized);
 #
 use WebDyne::Constant;
 use WebDyne::Err::Constant;
-use WebDyne::Base;
+use WebDyne::Util;
 
 
 #  External modules
 #
-use HTTP::Status qw(is_success is_error RC_INTERNAL_SERVER_ERROR);
+use HTTP::Status qw(:constants is_error);
 use File::Spec;
+use Data::Dumper;
+$Data::Dumper::Indent=1;
 
 
 #  Version information
 #
-$VERSION='1.250';
+$VERSION='2.074_566';
 
 
 #  Debug
@@ -48,7 +49,6 @@ debug("%s loaded, version $VERSION", __PACKAGE__);
 #  Package wide vars
 #
 my %Package;
-*debug=\&WebDyne::debug;
 
 
 #  Fix issues if mod_perl loads legacy Carp with modern Carp::Heavy
@@ -86,9 +86,9 @@ sub err_html {
     #  Get errstr from stack if not supplied, or add if it
     #  has been
     #
-    if ($errstr) {err ($errstr)}
+    if ($errstr) {err($errstr)}
     else {
-        $errstr=errstr() || do {err ($_='undefined error from handler'); $_}
+        $errstr=errstr() || do {err($_='undefined error from handler'); $_}
     }
 
     #$errstr ? err($errstr) : ($errstr=errstr() || do {err($_='undefined error from handler'); $_});
@@ -120,7 +120,7 @@ sub err_html {
     #  Try to get CGI object from class, or create if not present - may
     #  not have been initialised before error occured);
     #
-    my $cgi_or=$self->{'_CGI'} || CGI->new();
+    my $cgi_or=$self->{'_CGI'} || CGI::Simple->new($r);
     debug("cgi_or $cgi_or");
 
 
@@ -129,9 +129,11 @@ sub err_html {
     $r->log_error($errstr);
 
 
-    #  Status must be internal error
+    #  Status must be internal error if not set to something else already
     #
-    $r->status(RC_INTERNAL_SERVER_ERROR);
+    unless ($r->status() && (is_error($r->status()))) {
+        $r->status(HTTP_INTERNAL_SERVER_ERROR);
+    }
 
 
     #  Do not run any more handlers
@@ -139,8 +141,9 @@ sub err_html {
     $r->set_handlers(PerlHandler => undef);
 
 
-    #  Optionally kill this Apache process afterwards to make sure it does not behave
-    #  badly after this error, if that is what the user has configured
+    #  Optionally kill this Apache process afterwards to make sure it does
+    #  not behave badly after this error, if that is what the user has
+    #  configured
     #
     if ($WEBDYNE_ERROR_EXIT) {
         my $cr=sub {CORE::exit()};
@@ -159,8 +162,9 @@ sub err_html {
             "using text error (%s:%s:%s:%s) - update $r content_type",
             $WEBDYNE_ERROR_TEXT, $WEBDYNE_EVAL_SAFE, $self->{'_error_handler_run'}, $cgi_or
         );
-        $r->content_type('text/plain');
-
+        #$r->content_type('text/plain');
+        $r->content_type($WEBDYNE_CONTENT_TYPE_TEXT);
+        
 
         #  Push error
         #
@@ -168,6 +172,7 @@ sub err_html {
             {
 
                 'URI'  => $r->uri(),
+                #'Line' => scalar $self->data_ar_html_line_no(pop @{$self->{'_data_ar_err'}}),
                 'Line' => scalar $self->data_ar_html_line_no(),
 
             });
@@ -195,11 +200,11 @@ sub err_html {
         my @errstack=@{&errstack()};
         my %param=(
 
-            errstr      => $errstr,
-            errstack_ar => \@errstack,
-            errperl_sr  => $self->{'_err_perl_sr'},
-            data_ar     => $self->{'_data_ar'},
-            r           => $r
+            errstr           => $errstr,
+            errstack_ar      => \@errstack,
+            err_eval_perl_sr => $self->{'_err_eval_perl_sr'},
+            err_eval_line    => $self->{'_err_eval_line'},
+            data_ar_err      => $self->{'_data_ar_err'},
 
         );
 
@@ -222,9 +227,11 @@ sub err_html {
             local $SIG{__DIE__};
             require WebDyne::Compile;
             my $container_ar=(
-                $Package{'container_ar'} ||= &WebDyne::Compile::compile(
-                    $self,
-                    {
+
+                #  Don't cache it - only minor penalty to recompile and WEBDYNE_RELOAD=1 breaks error handler
+                #  if multiple errors.
+                #$Package{'container_ar'} ||= &WebDyne::Compile::compile(
+                $self->WebDyne::Compile::compile({
 
                         srce     => $WEBDYNE_ERR_TEMPLATE,
                         nofilter => 1
@@ -236,18 +243,24 @@ sub err_html {
             #  to use internal
             #
             my $data_ar=$container_ar->[$WEBDYNE_CONTAINER_DATA_IX];
+            debug("err_html data_ar: %s", Dumper($param{'data_ar'}));
 
 
             #  Reset render state and render error page
             #
             $self->render_reset($data_ar);
-            my $html_sr=$self->render(
-                {
+            #my $html_sr=$self->render({
+##
+#                    data  => $data_ar,
+#                    param => \%param
+#
+#            }) || return $self->err_html('fatal problem in error handler during render: %s !', errstr() || 'undefined error');
+            my $html_sr=$self->render_data_ar(
 
                     data  => $data_ar,
                     param => \%param
 
-                }) || return $self->err_html('fatal problem in error handler during render: %s !', errstr() || 'undefined error');
+            ) || return $self->err_html('fatal problem in error handler during render: %s !', errstr() || 'undefined error');
 
 
             #  Set custom handler
@@ -259,7 +272,7 @@ sub err_html {
 
             #  Clear error stack again, make sure all is clean before we return.
             #
-            errclr(); eval {undef} if $@;
+            errclr(); eval {} if $@;
 
         };
 
@@ -269,8 +282,8 @@ sub err_html {
         #
         if ($@ || !$status) {
             debug("unable to render HTML template, reverting to text");
-                err ($@) if $@;
-                err ('previous error stack %s', Data::Dumper::Dumper(\@errstack));
+            err($@) if $@;
+            err('previous error stack %s', Data::Dumper::Dumper(\@errstack));
             my $webdyne_error_text_save=$WEBDYNE_ERROR_TEXT;
             $WEBDYNE_ERROR_TEXT=1;
             $status=$self->err_html($errstr);
@@ -280,6 +293,7 @@ sub err_html {
 
         #  Return result
         #
+        debug("return status: $status");
         return $status
 
     }
@@ -291,13 +305,28 @@ sub err_eval {
 
     #  Special handler for eval errors
     #
-    my ($self, $message, $perl_sr)=@_;
+    my ($self, $message, $perl_sr, $inode)=@_;
     debug("err_eval $message, %s, caller %s", Dumper($perl_sr), Dumper([caller()]));
 
 
+    #  Try to scrape line from message
+    #
+    my ($err_eval_line)=($message=~/WebDyne::${inode}\s+line\s+(\d+)/);
+    unless ($err_eval_line) {
+
+        #  Only if Devel::Confess installed will this parse work
+        #
+        #  Illegal division by zero at (eval 211)[WebDyne::6d8da02b63e4707b80466fda560173a6:5] line 1.
+        #
+        ($err_eval_line)=($message=~/\[WebDyne::${inode}:(\d+)\]/);
+    }
+    debug("err_eval eval_line:$err_eval_line");
+
+
     #  Store away for future ref by error handler
     #
-    $self->{'_err_perl_sr'}=$perl_sr;
+    $self->{'_err_eval_perl_sr'}=$perl_sr;
+    $self->{'_err_eval_line'}=$err_eval_line;
 
 
     #  Send message off to main error handler and return
